//____________________________________________________________________________
/*!

\class   AanetWrite

\brief   A class to write gSeaGen output in a Aanet file 

\author  lfonso Andres Garcia Soto <alfonsog@nikhef.nl>
         Nikhef, Netherlands

\created 2019

\cpright  Copyright (c) 2019, The KM3NeT Collaboration 
          For the full text of the license see $GSEAGEN/LICENSE

*/
//____________________________________________________________________________


#ifdef _AANET_ENABLED__

#ifndef _xxAANETWRITE_H__
#define _xxAANETWRITE_H__

#include <cstdio>
#include <fstream>
#include <ctime>
#include <string>
#include <map>

#include <TSystem.h>

#include "PropaMuon/PropaTracks.h" 
#include "SeaEvent/GBinParam.h"
#include "SeaEvent/GSeaEvent.h"

#include <ostream>
#include "Evt.hh"
#include "EventFile.hh"
#include "TFile.h"
#include "stringutil.hh"

using namespace std;
using namespace genie;

class AanetWrite {

public :

  GenParam* fGenPar;
  TFile* fOutFile;
  TTree* fTree;
  Evt*   fEvt;

  AanetWrite(GenParam* GenPar ) {

    fGenPar = GenPar;
    
    ostringstream fnstr;
    fnstr << fGenPar->EvFilePrefix  << "." << fGenPar->RunNu << ".aa.root";
    
    string OutFileName=fnstr.str(); 
    
    // setup aanet output tree
    fEvt     = new Evt();
    fOutFile = new TFile( OutFileName.c_str() , "RECREATE");
    fTree    = new TTree( "E", "Aanet Evt Tree");
    fTree->Branch("Evt", &fEvt, 32000, 4 ); 
  }
  
  ~AanetWrite() {
    fOutFile->Write();
    delete fOutFile;
  }
  
  void WriteHeader() {

    string prefix = "";

    fOutFile->cd();
 
    Head h;   
    tm *date;
    date = localtime (&fGenPar->RunTime);
    
    h.get_line(prefix+"version")      = form("gSeaGen %s %2.2d%2.2d%2.2d  %2.2d%2.2d%2.2d ",
                                             fGenPar->gSeaGenVer.c_str(),
                                             date->tm_year-100,
                                             date->tm_mon+1,
                                             date->tm_mday,
                                             date->tm_hour,
                                             date->tm_min,
                                             date->tm_sec );
    h.get_line(prefix+"interaction")  = form("GENIE %s",
                                            fGenPar->GenieVer.c_str() );
    h.get_line(prefix+"drawing")      = fGenPar->Drawing;
    h.get_line(prefix+"cut_nu")       = form("%10.2E %10.2E %10.2E %10.2E",
                                             fGenPar->EvMin,
                                             fGenPar->EvMax,
                                             fGenPar->CtMin,
                                             fGenPar->CtMax);
    h.get_line(prefix+"spectrum")     = form("%8.2f", -fGenPar->Alpha);
    h.get_line(prefix+"seed")         = form("d", fGenPar->RanSeed );
    h.get_line(prefix+"xsecfile")     = fGenPar->InpXSecFile;
    h.get_line(prefix+"detector")     = fGenPar->detector;
    h.get_line(prefix+"propcode")     = fGenPar->PropCode;
    h.get_line(prefix+"tgen")         = form("%f", fGenPar->TGen );
    h.get_line(prefix+"source_mode")  = fGenPar->GenMode;
    if ( fGenPar->SourceName != "" )  h.get_line(prefix+"source_name") = fGenPar->SourceName;
    if ( fGenPar->GenMode == "POINT") {
      h.get_line(prefix+"mjd_interval")       = form("%f %f", fGenPar->MJDStart,fGenPar->MJDStop);
      h.get_line(prefix+"source_radius")      = form("%f", fGenPar->SourceRadius*180./M_PI);
      h.get_line(prefix+"source_coordinates") = form("%f %f",
                                                     fGenPar->RightAscension,
                                                     fGenPar->Declination);
    }
    stringstream oline;
    oline << fGenPar->Can1 <<" "<< fGenPar->Can2<< " " << fGenPar->Can3;
    h.get_line(prefix+"can")          = oline.str();
    h.get_line(prefix+"coord_origin") = form("10.3f %10.3f %10.3f",
                                             fGenPar->OriginX,
                                             fGenPar->OriginY,
                                             fGenPar->OriginZ );
    h.get_line(prefix+"coord_site")   = form("%10.3f %10.3f",
                                             fGenPar->SiteLatitude,
                                             fGenPar->SiteLongitude );
    h.get_line(prefix+"depth")        = form("%0.3f", fGenPar->SiteDepth);
    h.get_line(prefix+"bedrock")      = form("%10.3f", fGenPar->HBedRock );
    h.get_line(prefix+"genvol")       = form("%10.3f %10.3f %10.3f %12.2E %12.2E",
                                             fGenPar->HRock,
                                             fGenPar->HSeaWater,
                                             fGenPar->RVol,
                                             fGenPar->HSeaWater*fGenPar->RhoSW+fGenPar->HRock*fGenPar->RhoSR, // invol
                                             fGenPar->NTot );

    // AH: I am not understanding the fluxfiles output, so I don't
    // do it for the mometn
    
    h.Write("Header");
  
  }
    

  void write( GSeaTrack& gst, Trk& t ) {
    t.id        = gst.Id;
    t.status    = gst.Status;
    t.type      = gst.PdgCode;
    t.mother_id = gst.MotherId ;
    t.E         = gst.E;
    t.pos       = Vec( gst.V1, gst.V2, gst.V3 );
    t.dir       = Vec( gst.D1, gst.D2, gst.D3 );
    t.t         = gst.T;
  }

  Trk to_trk( GSeaTrack& gst ) {
    Trk t;
    write( gst, t );
    return t;
  }
  
  void WriteEvent(GSeaEvent * SeaEvt) {

    fEvt -> id = SeaEvt->iEvt;
    fEvt -> mc_trks.clear();
    for( auto gst : SeaEvt->Tracks ) fEvt->mc_trks.push_back ( to_trk( gst ));
    
    // weight information
    fEvt -> w = { SeaEvt->Agen, SeaEvt->GenWeight, SeaEvt->EvtWeight };

    int lepincan = SeaEvt->LepInCan ? 1 : 0;
    int verincan = SeaEvt->VerInCan;
      
    fEvt -> w2list = { fGenPar->GlobalGenWeight, //0
                       pow(SeaEvt->Neutrino.E,fGenPar->Alpha),          
                       SeaEvt->XSecMean,
                       SeaEvt->ColumnDepth,
                       SeaEvt->PEarth,
                       SeaEvt->WaterIntLen,
                       SeaEvt->PScale,                     
                       // AH: I add the ones below to the normal w2list
                       SeaEvt->Bx,
                       SeaEvt->By,
                       SeaEvt->WaterXSec,
                       SeaEvt->XSec,
                       SeaEvt->DXSec,
                       double( SeaEvt->ScattId ),        
                       double( SeaEvt->InterId ),
                       double( SeaEvt->TargetQrkPdg ),
                       double( verincan ),
                       double( lepincan ) };
    fTree->Fill();
  }
    
  
};

#endif // _GAANETWRITE_H__

#endif
