//____________________________________________________________________________
/*!

\class   genie::EvtWrite

\brief   A class to write gSeaGen output in a EVT file 

\author  Carla Distefano <distefano_c@lns.infn.it>
         LNS-INFN, Catania

\created May 22, 2012

\cpright  Copyright (c) 2012-2019, The KM3NeT Collaboration 
          For the full text of the license see $GSEAGEN/LICENSE

*/
//____________________________________________________________________________


#ifdef _ANTARES_ENABLED__

#include "OutputWriters/EvtWrite.h"

using namespace std;
using namespace genie;
using namespace genie::constants;


//___________________________________________________________________________
EvtWrite::EvtWrite(GenParam * GenPar, bool OptPDG)
{
  
  fGenPar = GenPar;
  
  fOptPDG = OptPDG;
  
  fNewEvt = true;
  
  ostringstream fnstr;
  fnstr << fGenPar->EvFilePrefix  << "." 
        << fGenPar->RunNu
        << ".evt";

  string OutFileName=fnstr.str(); 
  fOutFile.open(OutFileName.c_str());

  fPdg = new TDatabasePDG();
  
  fEvt = new event();

}

//___________________________________________________________________________
EvtWrite::EvtWrite(GenParam * GenPar, bool OptPDG, event *evt)
{
  fGenPar = GenPar;
  
  fOptPDG = OptPDG;
  
  fNewEvt = true;
  
  ostringstream fnstr;
  fnstr << fGenPar->EvFilePrefix  
//  	<< "." 
//        << fGenPar->RunNu
        << ".evt";

  string OutFileName=fnstr.str(); 
  fOutFile.open(OutFileName.c_str());

  fPdg = new TDatabasePDG();  
  
  fEvt = evt;
  fNewEvt = false;

}

//___________________________________________________________________________
EvtWrite::~EvtWrite(){
  fOutFile.close();
  if(fPdg) delete fPdg;
  if(fEvt) delete fEvt;
}

//___________________________________________________________________________
void EvtWrite::WriteHeader(void)
{

  tm *date;
  date = localtime (&fGenPar->RunTime);
  
  char ch[1024];

  if(fNewEvt)fEvt->defrun(fGenPar->RunNu);
  
  string CodVer=fGenPar->gSeaGenVer;
  if(CodVer.compare("dev")!=0){
    CodVer=CodVer.substr(1,3);
    CodVer.replace(1,1,".");
  }
  
  sprintf(ch, "gSeaGen %s %2.2d%2.2d%2.2d  %2.2d%2.2d%2.2d",CodVer.c_str(),
      date->tm_year-100,date->tm_mon+1,date->tm_mday,date->tm_hour,date->tm_min,date->tm_sec);
  fEvt->taga("simul",ch);

  sprintf(ch, "GENIE %s %2.2d%2.2d%2.2d  %2.2d%2.2d%2.2d",fGenPar->GenieVer.c_str(),
      date->tm_year-100,date->tm_mon+1,date->tm_mday,date->tm_hour,date->tm_min,date->tm_sec);
  fEvt->taga("simul",ch);

#ifdef _GENIEVERSIONGTEQ3__
  string ReWgtVer;
  string VerFile = gSystem->Getenv("GENIE_REWEIGHT");
  VerFile.append("/VERSION"); 
  ifstream file_stream(VerFile.c_str(), ios::in);     
  file_stream>>ReWgtVer;
  file_stream.close();

  sprintf(ch, "GENIE_REWEIGHT %s %2.2d%2.2d%2.2d  %2.2d%2.2d%2.2d",ReWgtVer.c_str(),
      date->tm_year-100,date->tm_mon+1,date->tm_mday,date->tm_hour,date->tm_min,date->tm_sec);
  fEvt->taga("simul",ch);
#endif

  if(gSystem->Getenv("GXMLPATH")){
    string gxmlpath=gSystem->Getenv("GXMLPATH");
    sprintf(ch, "%s",gxmlpath.c_str());
    fEvt->taga("GXMLPATH",ch);
  }

  fEvt->taga("drawing",fGenPar->Drawing);

  sprintf(ch, "%10.2E %10.2E %10.2E %10.2E", 
              fGenPar->EvMin,
	      fGenPar->EvMax,
	      fGenPar->CtMin,
	      fGenPar->CtMax);
  fEvt->taga("cut_nu",ch);

  sprintf(ch, "%8.2f", -fGenPar->Alpha);
  if(fNewEvt)fEvt->taga("spectrum",ch);

  sprintf(ch, "gSeaGen %d", fGenPar->RanSeed);
  fEvt->taga("seed",ch);

  if(fGenPar->InpXSecFile.size()>0){
    sprintf(ch, "%s", fGenPar->InpXSecFile.c_str());
    fEvt->taga("XSecFile",ch);
  }  


#ifdef _GENIEVERSIONGTEQ3__
  sprintf(ch, "%s", fGenPar->GenTune.c_str());
  fEvt->taga("tune",ch);
#endif

/*
  if(fGenPar->detector.size()>0){
    sprintf(ch, "%s 1 1", fGenPar->detector.c_str());
    fEvt->taga("detector",ch);
  }
*/
  if(fGenPar->TrackEvts){
    sprintf(ch, "%s", fGenPar->PropCode.c_str());
    fEvt->taga("prop_code",ch);
  }

  sprintf(ch, "%f", fGenPar->TGen);
  fEvt->taga("tgen",ch);

  sprintf(ch, "%s", fGenPar->GenMode.c_str());
  fEvt->taga("source_mode",ch);

  if(fGenPar->SourceName.size()>0){
    sprintf(ch, "%s", fGenPar->SourceName.c_str());
    fEvt->taga("source_name",ch);
  }
  
  if(fGenPar->GenMode.compare("POINT")==0){
    sprintf(ch, "%f %f", fGenPar->RightAscension,fGenPar->Declination);
    fEvt->taga("source_coordinates",ch);
    if(fGenPar->GenMJD){
      sprintf(ch, "%f %f", fGenPar->MJDStart,fGenPar->MJDStop);
      fEvt->taga("mjd_interval",ch);
    }
    sprintf(ch, "%f", fGenPar->SourceRadius*180./kPi);
    fEvt->taga("source_radius",ch);
  }

  ostringstream oline;
  oline << fGenPar->Can1 <<" "<< fGenPar->Can2<< " " << fGenPar->Can3;    
  string CanStr=oline.str();
//  fEvt->taga("can",CanStr);

  sprintf(ch, "%10.3f %10.3f %10.3f",
             fGenPar->OriginX,fGenPar->OriginY,fGenPar->OriginZ);
  fEvt->taga("coord_origin",ch); 

  sprintf(ch, "%10.3f %10.3f %10.3f %10.3f %10.3f",
             fGenPar->OriginX,fGenPar->OriginY,fGenPar->Can1-fGenPar->OriginZ,fGenPar->Can2-fGenPar->OriginZ,fGenPar->Can3);
  fEvt->taga("fixedcan",ch); 



  sprintf(ch, "%10.3f %10.3f",
             fGenPar->SiteLatitude,fGenPar->SiteLongitude);
  fEvt->taga("coord_site",ch); 


  sprintf(ch, "%10.3f", fGenPar->SiteDepth);
  fEvt->taga("depth",ch);  
  
  sprintf(ch, "%10.3f", fGenPar->HBedRock);
  fEvt->taga("bedrock",ch);
  
  
  double IntVol=constants::kPi*pow(fGenPar->RVol,2);
  IntVol *= fGenPar->HSeaWater*fGenPar->RhoSW+fGenPar->HRock*fGenPar->RhoSR;


  sprintf(ch, "%10.3f %10.3f %10.3f %12.2E %12.2E",
             fGenPar->HRock,fGenPar->HSeaWater,fGenPar->RVol,IntVol,fGenPar->NTot);
  fEvt->taga("genvol",ch); 

  for(unsigned iFlux=0; iFlux<fGenPar->FluxFiles.size(); iFlux++){
    ostringstream fline;    
    fline<<fGenPar->FluxFiles[iFlux].NuType<<" "<<fGenPar->FluxFiles[iFlux].FluxSimul<<" ";    
    for(unsigned iFile=0; iFile<fGenPar->FluxFiles[iFlux].FileNames.size(); iFile++)
      fline<<fGenPar->FluxFiles[iFlux].FileNames[iFile]<<" ";
    
    string flux=fline.str();
    fEvt->taga("flux",flux);
  }

  fEvt->write(fOutFile);  

  return;
}
//________________________________________________________________________________________

void EvtWrite::WriteEvent(GSeaEvent * SeaEvt){


  char ch[1024];

  int Geant3Code;
  int PdgCode;
  int WriteCode;

  // Get a RandomGen instance
  RandomGen * rnd = RandomGen::Instance();

  if(fNewEvt)fEvt->defeve(SeaEvt->iEvt,1);

  if(SeaEvt->Neutrino.Id!=0){

  sprintf(ch, "%d %12.3f %12.3f %12.3f %12.6f %12.6f %12.6f %12.4E %12.6f %12.6f %12.6f %d %d %d", 
               SeaEvt->Neutrino.Id,
	       SeaEvt->Neutrino.V1, 
	       SeaEvt->Neutrino.V2, 
	       SeaEvt->Neutrino.V3, 
	       SeaEvt->Neutrino.D1, 
	       SeaEvt->Neutrino.D2, 
	       SeaEvt->Neutrino.D3, 
	       SeaEvt->Neutrino.E, 
	       SeaEvt->Neutrino.T,	       
	       SeaEvt->Bx,SeaEvt->By,SeaEvt->ScattId,	       
	       SeaEvt->Neutrino.PdgCode,
	       SeaEvt->InterId);	
  fEvt->taga("neutrino",ch);  

  // if neutral current, neutrinos have pdg code in the primarylepton code (and not geant3 code)
  PdgCode=SeaEvt->PrimaryLepton.PdgCode; 
  Geant3Code=fPdg->ConvertPdgToGeant3(PdgCode);     
    
  // if neutral current, neutrinos have pdg code in the primarylepton code (and not geant3 code)    
  if(fOptPDG || SeaEvt->InterId==3)WriteCode = PdgCode;
  else WriteCode = Geant3Code;
    
  sprintf(ch, "%d %12.3f %12.3f %12.3f %12.6f %12.6f %12.6f %12.4E %12.6f %d", 
               SeaEvt->PrimaryLepton.Id, 
	       SeaEvt->PrimaryLepton.V1, 
	       SeaEvt->PrimaryLepton.V2, 
	       SeaEvt->PrimaryLepton.V3, 
	       SeaEvt->PrimaryLepton.D1, 
	       SeaEvt->PrimaryLepton.D2, 
	       SeaEvt->PrimaryLepton.D3, 
	       SeaEvt->PrimaryLepton.E, 
	       SeaEvt->PrimaryLepton.T,	       	     
	       WriteCode);	       
	       
  fEvt->taga("primarylepton",ch);  


//  sprintf(ch, "%s %12.3f",SeaEvt->TargetName.c_str(),SeaEvt->TargetMass);  
  sprintf(ch, "%d %d",SeaEvt->TargetZ,SeaEvt->TargetA);    
  fEvt->taga("target",ch);



  sprintf(ch, "%12.3E %12.3E %12.3E", SeaEvt->Agen, SeaEvt->GenWeight, SeaEvt->EvtWeight);
  fEvt->taga("weights",ch);
  
  sprintf(ch, "%12.3E %12.3E %12.3E %12.3E %12.4E %12.3E %12.3E",
              fGenPar->GlobalGenWeight, pow(SeaEvt->Neutrino.E,fGenPar->Alpha),SeaEvt->XSecMean,SeaEvt->ColumnDepth,SeaEvt->PEarth,SeaEvt->WaterIntLen,SeaEvt->PScale);
  fEvt->taga("w2list",ch);
  
  sprintf(ch, "%d", SeaEvt->VerInCan);
  fEvt->taga("vertex",ch);

  }






  for(unsigned iTrack=0; iTrack<SeaEvt->Tracks.size(); iTrack++){
  
    PdgCode=SeaEvt->Tracks[iTrack].PdgCode;
    if(TMath::Abs(PdgCode)==311){ // K0 (K0Bar) "decays" in K0S or KOL	  
      if(rnd->RndFlux().Rndm()<=0.5)PdgCode=130;
      else PdgCode=310;	  
    }
    // all neutrinos and anti-neutrinos have Geant3Code=4  
    if(TMath::Abs(PdgCode)==12 || TMath::Abs(PdgCode)==14 || TMath::Abs(PdgCode)==16)Geant3Code=4;
    else Geant3Code=fPdg->ConvertPdgToGeant3(PdgCode);
	
    if(fOptPDG)WriteCode = PdgCode;
    else WriteCode = Geant3Code;

    if(SeaEvt->Tracks[iTrack].Status>0 && PdgCode<1000000000){ // exclude probe neutrino, target and remnant nuclei, binding energy (pdg=2000000101)
      sprintf(ch, "%3d %12.3f %12.3f %12.3f %12.6f %12.6f %12.6f %12.4E %15.6f %d", 
               SeaEvt->Tracks[iTrack].Id, 
	       SeaEvt->Tracks[iTrack].V1, 
	       SeaEvt->Tracks[iTrack].V2, 
	       SeaEvt->Tracks[iTrack].V3, 
	       SeaEvt->Tracks[iTrack].D1, 
	       SeaEvt->Tracks[iTrack].D2, 
	       SeaEvt->Tracks[iTrack].D3, 
	       SeaEvt->Tracks[iTrack].E, 
	       SeaEvt->Tracks[iTrack].T,	    
	       WriteCode);	        
      
      if(SeaEvt->Tracks[iTrack].Status>1000)fEvt->taga("track_earthlepton",ch);
      else if(SeaEvt->Tracks[iTrack].Status%100==1){
        if(Geant3Code==0)cout<<"No Geant3 Code for PDG "<<PdgCode<<endl;
        if (SeaEvt->Tracks[iTrack].InCan!=0)fEvt->taga("track_in",ch);
      }

    }

  }

  
  

  if(gSystem->Getenv("SYSTLIST")){
    ostringstream fline;
    fline<<SeaEvt->SysWgt.WList.size()<<" ";
    for(unsigned i=0; i<SeaEvt->SysWgt.WList.size(); i++)fline<<SeaEvt->SysWgt.WList[i]<<" ";
    fEvt->taga("wSysGen",fline.str());
    for(unsigned i=0; i<SeaEvt->SysWgt.WParam.size(); i++){
      fline.str("");
      fline<<SeaEvt->SysWgt.WParam[i].wght.size()<<" ";    
      string tag="wSysGen_";
      for(unsigned j=0; j<SeaEvt->SysWgt.WParam[i].wght.size(); j++)fline<<SeaEvt->SysWgt.WParam[i].wght[j]<<" ";
        tag.append(SeaEvt->SysWgt.WParam[i].name);
        fEvt->taga(tag.c_str(),fline.str());
      }
  }


  
  if(fGenPar->GenMode.compare("POINT")==0){
    sprintf(ch, "%f", SeaEvt->LST);
    fEvt->taga("LST",ch);
    if(fGenPar->GenMJD){
      sprintf(ch, "%f ", SeaEvt->MJD);
      fEvt->taga("MJD",ch);
    }
  }

  fEvt->write(fOutFile);  

  return;
}

#endif
