//____________________________________________________________________________
/*!

\class   genie::BinParam, genie::GenParam 

\brief   Data structure for gSeaGen

\author  Carla Distefano <distefano_c@lns.infn.it>
         LNS-INFN, Catania
         Included modifications by Alfonso Garcia <alfonsog@nikhef.nl>
	 NIKHEF, Amsterdam (January 2019)

\created May 13, 2012

\cpright  Copyright (c) 2012-2019, The KM3NeT Collaboration
          For the full text of the license see $GSEAGEN/LICENSE

*/
//____________________________________________________________________________


#ifndef _GBINPARAM_H__
#define _GBINPARAM_H__

#include "TGraph.h"
#include "TDatime.h"
#include "TRotation.h"

#include "SeaNuDrivers/GSeaSystParam.h"

#include <map>

using namespace std;

struct BinParam {
  double NEvBin, NEvBinSW, NEvBinSR; 
  int NEvBinScal, NEvBinGen; 
  int NEvBinScalSW, NEvBinGenSW; 
  int NEvBinScalSR, NEvBinGenSR; 
  double EneMin, EneMax, RT, RL;
  double X0,Y0,Z0;
  double ProbScale, ProbScaleSW, ProbScaleSR; 
  double RVol, HSeaWater, HRock, RockCenter, SeaCenter;
  string geometry;
};

class FluxFileStr : public TObject {

  public:
  int NuType;
  string FluxSimul;
  vector<string> FileNames;
  
  FluxFileStr(){};
  ~FluxFileStr(){};
  
  ClassDef(FluxFileStr,1);
  
  
};

// GenParam class contains all inputs used in the simulation. 
// Some of these inputs are computed by the code itself (e.g. cross sections enetring in the weights)
// Members of this class are filled by different classes at initializzation

class GenParam : public TObject {
  
  public:
 
  int RunNu;				// Run number 
  int RanSeed;      			// random number seed
  string EventGeneratorList;            // list of interaction channels
  string InpXSecFile;  			// cross-section splines
  double NTot;				// Total number of generated neutrinos
  double EvMin, EvMax;			// Generated neutrino energy range
  double CtMin, CtMax;                  // Generated neutrino cosTheta range
  double Alpha;                   	// Generation spectral index
  int NBin;				// number of energy bins
  double Can1, Can2, Can3;  		// Zmin Zmax and Rad of detector can
  double OriginX, OriginY, OriginZ;	// Coordinates of can orgin in the detector reference system in meters
  double HRock, HSeaWater, RVol;	// interaction volume: height in rock and water and radius
  double HBedRock;			// height of the bedrock to add below the can to take into account that some particles may escape into the ocean and create visible Cherenkov light
  double NAbsLength;
  double AbsLength;
  double SiteDepth;
  double SiteLatitude;			// detector latitude in rad
  double SiteLongitude;			// detector longitude in rad
  double SeaBottomRadius;
  double GlobalGenWeight;
  double RhoSW, RhoSR;
  double TGen;				// simulated interval time in seconds

  bool   TrackEvts;			// if true propagator code is initialized

  int    WriteParticlesMode;            // 0,1,2 for increasingly more particles  

  int   WeightOpt;   //0,1,2 wieghting options in GMCJDriver (GENIE)

  string RTOpt;				// RT definition opt: can, vol or set by hand 

  string GenCode;			// if genie or other generator
  string PropCode;
  
  string GenMode;			// DIFFUSE or POINT
  string SourceFile;
  bool   GenMJD;
  
  string SourceName;
  double Declination,RightAscension,SourceRadius;
  double MJDStart, MJDStop;

  map<int,double> SeaWaterComp, RockComp, MantleComp, CoreComp;
  map<int, TGraph> XSecWater;

  map<int, TGraph> XSecTotSW;         		///< CC cross sections per nucleon in m^2 (seawater)
  map<int, TGraph> XSecTotSR;         		///< CC cross sections per nucleon in m^2 (rock/crust)
  map<int, TGraph> XSecTotMantle;      		///< CC cross sections per nucleon in m^2 (mantle)
  map<int, TGraph> XSecTotCore;        		///< CC cross sections per nucleon in m^2 (core)

  double MolarMassSW, MolarMassSR, MolarMassMantle, MolarMassCore;   // molar mass of target media
  int NNucleonsSW, NNucleonsSR, NNucleonsMantle, NNucleonsCore;	     // number of nucleons in target media
  int NElectronsSW, NElectronsSR, NElectronsMantle, NElectronsCore;  // number of electrons in target media

  
  TGraph RangeSW,RangeSR;
  string detector;			// detector geometry file (for can definition)
  
  vector<int> NuList;			// list of generated neutrino type  
  vector<FluxFileStr> FluxFiles;
 
  TRotation Rotation;                   // coordinate rotation matrix: topocentric horizontal -> user-defined topocentric system

  string EvFilePrefix;
  
  bool NativeOutput;			// if true write the native generator output file
 
  bool PropMode;			// true if propagation through the Earth
  string Drawing;			// if surface or volume drawing
  
  string gSeaGenVer;			// code version for gSeaGen
  string GenieVer;			// code version for GENIE
  string GenTune;			// generator tune, required for genie version >=3
  
  time_t RunTime;			// running time
  
  
  SystInput SystParams;
  
  GenParam(){};
  ~GenParam(){};
  
  ClassDef(GenParam,1);
  
};



#endif // _GBINPARAM_H__

