//____________________________________________________________________________
/*!

\class   GAstro 

\brief   A class for astronomical calculation 

\author  Carla Distefano <distefano_c@lns.infn.it>
         LNS-INFN, Catania

\created February 1, 2016

\cpright  Copyright (c) 2016-2019, The KM3NeT Collaboration
          For the full text of the license see $GSEAGEN/LICENSE
*/
//____________________________________________________________________________


#include <fstream>
#include <cmath>
#include <iostream>
#include <fstream>
#include <string>
#include <cstdio>
#include <sstream>
#include <stdlib.h>
#include <cmath>
#include <cstring>


#ifdef _GENIEVERSIONGTEQ3__
#include "Framework/Conventions/Constants.h"
#include "Framework/Messenger/Messenger.h"
#include "Framework/Utils/StringUtils.h"
#else
#include "Conventions/Constants.h"
#include "Messenger/Messenger.h"
#include "Utils/StringUtils.h"
#endif

#include "SeaNuDrivers/GAstro.h"

using namespace std;
using namespace genie; 
using namespace genie::constants;


const double  Deg2Rad = kPi/180.;
const double  Rad2Hour= 12./kPi;


GAstro::GAstro(double Latitude, double Longitude){ 
// The longitude is positive westwards from the meridian of Greenwich, and negative to the east.

  fLatitude = Latitude;
  fLongitude = Longitude;
  
  LOG("GAstro", pINFO)<< "Initialization of GAstro with Latitude = "<<fLatitude<<", Longitude ="<<fLongitude;  
}
//___________________________________________________________________________
GAstro::~GAstro(){  }
//___________________________________________________________________________
void GAstro::SetCoordinates(double Latitude, double Longitude){
// The longitude is positive westwards from the meridian of Greenwich, and negative to the east.

  fLatitude = Latitude;
  fLongitude = Longitude;
  return;
}
//___________________________________________________________________________
double GAstro::GST(double MJD){  // Greenwich Sidereal Time as a function of the Modified Julian Day		
  double gst,T;
  T=(MJD-51544.5)/36525.0;
  gst=(280.46061837+13185000.77*T+(pow(T,2)*(0.000387933-T/38710000.0)))/15.;  
  gst=fmod(gst,24.0); //normalization between 0 e 24 hours 
  return gst/Rad2Hour; // in radians
}
//___________________________________________________________________________
double GAstro::LST(double MJD){ // Local Sidereal Time as a function of the Modified Julina Day and the observer's longitude  // dire come si deve calcolare la longitudine
  double lst,gst;  
  gst=this->GST(MJD);
  lst=gst+fLongitude; 	
  lst=fmod(lst,2*kPi);  	//normalization between 0 e 24 hours	
  return lst;  			// in radians
}
//___________________________________________________________________________
void GAstro::Eq2Hor(double lst, double RA, double Dec, double * Azimuth, double * altitude){ // transforms equatorial coordinates into horizontal ones
  double HA=lst-RA; //hour angle in radians
  *altitude =  asin(sin(fLatitude)*sin(Dec)+cos(fLatitude)*cos(Dec)*cos(HA)); 
  *Azimuth=atan2(sin(HA),(cos(HA)*sin(fLatitude)-tan(Dec)*cos(fLatitude)))+kPi; // from North
  *Azimuth=fmod(*Azimuth+2*kPi,2*kPi);
  return;	
}
//___________________________________________________________________________
void GAstro::Hor2Eq(double Altitude, double Azimuth, double lst, double * RA, double * Dec){ // transforms horizontal into equatorial ones
  Azimuth-=kPi;  // azimth is from north, now from south
  *Dec=asin(sin(fLatitude)*sin(Altitude)-cos(fLatitude)*cos(Altitude)*cos(Azimuth));
  double HA=atan2(sin(Azimuth),cos(Azimuth)*sin(fLatitude)+tan(Altitude)*cos(fLatitude));
  *RA=lst-HA;  
  *RA=fmod(*RA+2*kPi,2*kPi);
  return;
}
//___________________________________________________________________________
void GAstro::Gal2Eq(string Equinox, double GalLat, double GalLong, double * RA, double * Dec){// transforms galactic coordinates into equatorial ones

  double PoleRA=0.,PoleDec=0.,PosAngle=0.;
  double CentreRA=0., CentreDec=0.;

  if(Equinox.compare("J2000")==0){
    PoleRA = 192.859508;
    PoleDec = 27.128336;
    CentreRA = 266.4;
    CentreDec = -28.94;
  }
  else if(Equinox.compare("B1950")==0){
    PoleRA = 192.25;
    PoleDec = 27.4;
    CentreRA = 265.6;
    CentreDec = -28.92;
  }
  else {
    LOG("GAstro", pFATAL)<< "Unknown Equinox, it must be J2000 or B1950, exiting...";  
    gAbortingInErr = true;
    exit(1);           
  }
  
  PoleRA *= Deg2Rad;
  PoleDec *= Deg2Rad;
  CentreRA *= Deg2Rad;
  CentreDec *= Deg2Rad;

  PosAngle = acos(sin(CentreDec)*cos(PoleDec)-cos(CentreDec)*sin(PoleDec)*cos(PoleRA-CentreRA));
 
  *Dec = asin(sin(PoleDec)*sin(GalLat)+cos(PoleDec)*cos(GalLat)*cos(PosAngle-GalLong));
  *RA = atan2(cos(GalLat)*sin(PosAngle-GalLong),cos(PoleDec)*sin(GalLat)-sin(PoleDec)*cos(GalLat)*cos(PosAngle-GalLong))+PoleRA;
  *RA=fmod(*RA+2*kPi,2*kPi);

  return;
}
//___________________________________________________________________________
void GAstro::Eq2Gal(string Equinox, double RA, double Dec, double * GalLat, double * GalLong){// transforms equatorial coordinates into galactic ones

  double PoleRA=0.,PoleDec=0.,PosAngle=0.;
  double CentreRA=0., CentreDec=0.;

  if(Equinox.compare("J2000")==0){
    PoleRA = 192.859508;
    PoleDec = 27.128336;
    CentreRA = 266.4;
    CentreDec = -28.94;
  }
  else if(Equinox.compare("B1950")==0){
    PoleRA = 192.25;
    PoleDec = 27.4;
    CentreRA = 265.6;
    CentreDec = -28.92;
  }
  else {
    LOG("GAstro", pFATAL)<< "Unknown Equinox, it must be J2000 or B1950, exiting...";  
    gAbortingInErr = true;
    exit(1);           
  }
  
  PoleRA *= Deg2Rad;
  PoleDec *= Deg2Rad;
  CentreRA *= Deg2Rad;
  CentreDec *= Deg2Rad;

  PosAngle = acos(sin(CentreDec)*cos(PoleDec)-cos(CentreDec)*sin(PoleDec)*cos(PoleRA-CentreRA));
  
  *GalLat = asin(sin(PoleDec)*sin(Dec)+cos(PoleDec)*cos(Dec)*cos(RA-PoleRA));
  *GalLong = PosAngle-atan2(cos(Dec)*sin(RA-PoleRA),cos(PoleDec)*sin(Dec)-sin(PoleDec)*cos(Dec)*cos(RA-PoleRA));
  *GalLong=fmod(*GalLong+2*kPi,2*kPi);

  return;
}
//___________________________________________________________________________
void GAstro::Hor2Direction(double Azimuth, double altitude, double * vx, double * vy, double * vz){ // computes direction cosines corresponding to the source horizontal coordinates 

  // source theta and phi angles in radians
  double AstroTheta= kPi/2.-altitude;
  double AstroPhi=2*kPi-Azimuth;
  
  // source direction cosines
  *vx=sin(AstroTheta)*cos(AstroPhi);
  *vy=sin(AstroTheta)*sin(AstroPhi);
  *vz=cos(AstroTheta);
  
  return;
}
//___________________________________________________________________________
void GAstro::Direction2Hor(double vx, double vy, double vz,double * Azimuth, double * Altitude){ // computes source horzontal coordinates from the corresponging direction cosines

  // source theta and phi angles in radians
  double AstroTheta= acos(vz);
  double AstroPhi=atan2(vy,vx);

  *Altitude=kPi/2.-AstroTheta;
  *Azimuth=2*kPi-AstroPhi;  
  *Azimuth=fmod(*Azimuth+2*kPi,2*kPi);

  return;
}
//___________________________________________________________________________
double GAstro::CalcMJD(int M, int day, int Dh, int Dm, int Ds, int Y){ //computes the Modified Julian Day

  double D;
  int A,B;
  double MJD;

  D=(double)day+((double)Dh+(double)Dm/60.+(double)Ds/3600.)/24.;

  if(M == 1 || M  == 2){
    Y=Y-1;
    M=M+12;
  }
	
  A = (int)(Y/100);
  B = 2 - A +(int)(A / 4);
  MJD = (int)(365.25*Y) + (int)(30.6001*(M+1)) + D + 1720994.5 + B;  // Julian day
  MJD-=2400000.5; 

  return MJD;
}
//___________________________________________________________________________
double GAstro::CalcMJD(string Date){ //computes the Modified Julian Day
  // Date in format dd-mm-yyyy;hh:mm:ss
   
  int M, day, Dh, Dm, Ds, Y;
  bool FormatOk = true;
  
  if(Date.size()!=19)FormatOk=false;

  if(Date.substr(2,1)!="-")FormatOk=false;
  if(Date.substr(5,1)!="-")FormatOk=false;
  if(Date.substr(10,1)!=";")FormatOk=false;
  if(Date.substr(13,1)!=":")FormatOk=false;
  if(Date.substr(16,1)!=":")FormatOk=false;

  if(!FormatOk){
    LOG("GAstro", pFATAL)<< "Wrong format for Date "<<Date<<". It must be dd-mm-yyyy;hh:mm:ss, exiting...";  
    gAbortingInErr = true;
    exit(1);           
  }

  string::size_type semicolon = Date.find_first_of(";",0); 
  string Date1 = Date.substr(0,semicolon);
  Date.erase(0,semicolon+1);	
  
  vector<string> vDate = utils::str::Split(Date1, "-");
  assert(vDate.size() == 3);
  	
  day  = atoi(vDate[0].c_str());
  M    = atoi(vDate[1].c_str());
  Y    = atoi(vDate[2].c_str());
  
  vDate = utils::str::Split(Date, ":");
  assert(vDate.size() == 3);
  	
  Dh  = atoi(vDate[0].c_str());
  Dm  = atoi(vDate[1].c_str());
  Ds  = atoi(vDate[2].c_str());
  
  double MJD=this->CalcMJD(M, day, Dh, Dm, Ds, Y);

  return MJD;
}
//___________________________________________________________________________
void GAstro::CalcPartDirMJD(double MJD, double RA, double Dec, double * vx, double * vy, double * vz){ 
// computes the direction cosines of a particle from a source as a function of the MJD, observer's coordinates and source equatorial coordinates
  double lst;
  lst=this->LST(MJD);
  this->CalcPartDirLST(lst, RA, Dec, vx, vy, vz);
  return;
}
//___________________________________________________________________________
void GAstro::CalcPartDirMJD(double MJD, double RA, double Dec, double * CosTheta, double * Phi){ 
// computes the direction angles of a particle from a source as a function of the MJD, observer's coordinates and source equatorial coordinates
  
  double vx,vy,vz;
  this->CalcPartDirMJD(MJD, RA, Dec, &vx, &vy, &vz);
  *CosTheta=vz;
  *Phi=atan2(vy,vy);
  
  return;
}
//___________________________________________________________________________
void GAstro::CalcPartDirLST(double lst, double RA, double Dec, double * vx, double * vy, double * vz){ 
// computes the direction cosines of a particle from a source as a function of the MJD, observer's coordinates and source equatorial coordinates

  double Azimuth, altitude;
  this->Eq2Hor(lst, RA, Dec, &Azimuth,&altitude);
  this->Hor2Direction(Azimuth,altitude, vx,vy,vz);
//  cout<<"source: "<<LST<<" "<<Azimuth<<" "<<altitude<<" "<<-*vx<<" "<<-*vy<<" "<<-*vz<<endl;  
  return;
}
//___________________________________________________________________________
void GAstro::CalcPartDirLST(double lst, double RA, double Dec, double * CosTheta, double * Phi){ 
// computes the direction angles of a particle from a source as a function of the MJD, observer's coordinates and source equatorial coordinates
  
  double vx,vy,vz;
  this->CalcPartDirLST(lst, RA, Dec, &vx, &vy, &vz);
  *CosTheta=vz;
  *Phi=atan2(vy,vy);
  
  return;
}




