//____________________________________________________________________________
/*!
 
\class   genie::GGenerateEvent.cxx

\brief   A base generation driver for an under-water neutrino telescope 

\author  Carla Distefano <distefano_c@lns.infn.it>
         LNS-INFN, Catania
         Included modifications by Alfonso Garcia <alfonsog@nikhef.nl>
	 NIKHEF, Amsterdam (January 2019)

\created July 19, 2017

\cpright  Copyright (c) 2017-2019, The KM3NeT Collaboration
          For the full text of the license see $GSEAGEN/LICENSE

*/
//____________________________________________________________________________


#ifdef _GENIEVERSIONGTEQ3__
#include "Framework/Conventions/Constants.h"
#else
#include "Conventions/Constants.h"
#endif



#include "SeaNuDrivers/GGenerateEvent.h"



#ifdef _MUSIC_ENABLED__
extern "C" 	 	{ void initialize_music_sr_(int *muin);}
extern "C" 	 	{ void initialize_music_sw_(int *muin);}
extern "C"              { void muon_transport_sr_(double *VX,double *VY,double *VZ,double *CX,double *CY,double *CZ,double *E,double *Depth,double *T,double *Rho,int *flag1,int *flag2,double *VXF,double *VYF,double *VZF,double *CXF,double *CYF,double *CZF,double *EF,double *DepthF,double *TF);}
extern "C"              { void muon_transport_sw_(double *VX,double *VY,double *VZ,double *CX,double *CY,double *CZ,double *E,double *Depth,double *T,double *Rho,int *flag1,int *flag2,double *VXF,double *VYF,double *VZF,double *CXF,double *CYF,double *CZF,double *EF,double *DepthF,double *TF);}
#endif

//___________________________________________________________________________
void copy( const GSeaTrack& from , PropaTrack& to )
{
  to.Id             = from.Id;
  to.Status         = from.Status;
  to.Pdg            = from.PdgCode;
  to.MotherId       = from.MotherId; 
  to.InCan          = from.InCan; 
  to.D1             = from.D1;
  to.D2             = from.D2;
  to.D3             = from.D3;
  to.V1             = from.V1;
  to.V2             = from.V2;
  to.V3             = from.V3;
  to.E              = from.E;  
  to.T              = from.T; 
}

//___________________________________________________________________________
void copy( const  PropaTrack& from, GSeaTrack& to)
{
  to.Id             = from.Id;
  to.Status         = from.Status;
  to.PdgCode        = from.Pdg;
  to.MotherId       = from.MotherId; 
  to.InCan          = from.InCan; 
  to.D1             = from.D1;
  to.D2             = from.D2;
  to.D3             = from.D3;
  to.V1             = from.V1;
  to.V2             = from.V2;
  to.V3             = from.V3;
  to.E              = from.E;  
  to.T              = from.T; 
}



//___________________________________________________________________________
GGenerateEvent::GGenerateEvent(GenParam * GenPar)
{
  fGenPar = GenPar;
  fSeaEvent = new GSeaEvent();
  fNEvt=0;
  fNTot=0;

  EoF = false;

  // get flux driver
  flux_driver_diffuse=0;
  flux_driver_point=0;
  fFluxDriver=0; 
  fFileDriver=0;
  
#ifdef _ANTARES_ENABLED__  
  fAntaresEvt=0;
#endif  
  
  
  if(fGenPar->GenMode.compare("DIFFUSE")==0){
    flux_driver_diffuse = this->GetFluxDiffuse();
    fFluxDriver=dynamic_cast<GSeaAtmoFlux *>(flux_driver_diffuse);
  }
  else if(fGenPar->GenMode.compare("POINT")==0) {
    flux_driver_point = this->GetFluxPoint();   
    fFluxDriver=dynamic_cast<GSeaAtmoFlux *>(flux_driver_point);    
  }
#ifdef _ANTARES_ENABLED__   
  else if(fGenPar->GenMode.compare("FILE")==0){
    GSeaAntaresFileFlux * antares_file_driver = new GSeaAntaresFileFlux(fGenPar);
    fFileDriver = dynamic_cast<GSeaFileFlux *>(antares_file_driver);
    fAntaresEvt=antares_file_driver->GetEvent();
  }
#endif
  else {
    LOG("GGenerateEvent", pFATAL) << "Unknown generation mode "<<fGenPar->GenMode<<", exiting...";
    exit(1);
  }


  if(fGenPar->TrackEvts){	// we initialize the propagator only if we use them
    if(fGenPar->PropCode.compare("PropaMuon")==0){
      PropaMuonSW = new PropaMuon(fGenPar->SeaWaterComp,fGenPar->RhoSW);
      PropaMuonSR = new PropaMuon(fGenPar->RockComp,fGenPar->RhoSR);   
    }
#ifdef _MUSIC_ENABLED__    
   else if(fGenPar->PropCode.compare("MUSIC")==0){
     string music_path;
     if(gSystem->Getenv("MUSICPATH"))music_path = gSystem->Getenv("MUSICPATH");
     else {
       music_path = gSystem->Getenv("GSEAGEN");
       music_path.append("/dat");   
       
       music_path = BOOST_PP_STRINGIZE(_MUSICDATA__);
       while(1){
         size_t pos = music_path.find("\"");
         if(pos==string::npos)break;
         music_path.erase(pos,1);
       }
        
     }
     string cp_command("cp ");
     cp_command.append(music_path);
     cp_command.append("/music*.dat .");  
     system(cp_command.c_str());
      
     int muin =2;
     initialize_music_sr_(&muin);
     initialize_music_sw_(&muin);
  } 
#endif  
#ifdef _PROPOSAL_ENABLED__    
   else if(fGenPar->PropCode.compare("PROPOSAL")==0){
  
     Medium *med_sr = new Medium("standard_rock",1.);
     Medium *med_sw = new Medium("water",1.);
   
     EnergyCutSettings* cuts = new EnergyCutSettings(1000,0.01);
  
     string table_path=PROPOSALRES;
     table_path.append("/tables");
    
     ProposalSW = new Propagator(med_sw,cuts,"mu",table_path.c_str());    
     ProposalSR = new Propagator(med_sr,cuts,"mu",table_path.c_str());
  
     ProposalSW->set_seed(fGenPar->RanSeed);
     ProposalSW->RandomDouble();  

     ProposalSR->set_seed(fGenPar->RanSeed);
     ProposalSR->RandomDouble();  
   }
#endif
  }

}
//___________________________________________________________________________
GGenerateEvent::~GGenerateEvent(){


  if(fFluxDriver)delete fFluxDriver;
#ifdef _ANTARES_ENABLED__   
  if(fFileDriver)delete fFileDriver;
#endif

  if(fGenPar->TrackEvts){
    if(fGenPar->PropCode.compare("PropaMuon")==0){
      if(PropaMuonSW)delete PropaMuonSW;
      if(PropaMuonSR)delete PropaMuonSR;
    }
#ifdef _PROPOSAL_ENABLED__     
    if(fGenPar->PropCode.compare("PROPOSAL")==0){
      if(ProposalSW)delete ProposalSW;
      if(ProposalSR)delete ProposalSR;
    }
#endif    
  }

  if(fSeaEvent)delete fSeaEvent;

}

//________________________________________________________________________________________
GSeaRealAtmoFlux * GGenerateEvent::GetFluxDiffuse(void)
{

  // Instantiate the diffuse flux driver

//  cout<<"Initialize GSeaRealAtmoFlux"<<endl;
  GSeaRealAtmoFlux * atmo_flux_driver = new GSeaRealAtmoFlux(fGenPar);

  // set rotation for coordinate tranformation from the topocentric horizontal
  // system to a user-defined coordinate system:
  
  if(!fGenPar->Rotation.IsIdentity()) {
     atmo_flux_driver->SetUserCoordSystem(fGenPar->Rotation);
  }

//  cout<<"Initialize GetGlobalGenWeight"<<endl;  
  fGenPar->GlobalGenWeight=atmo_flux_driver->GetGlobalGenWeight();

  return atmo_flux_driver;
}
//________________________________________________________________________________________
GSeaRealPointFlux* GGenerateEvent::GetFluxPoint(void)
{
  // Instantiate the point flux driver
  
  GSeaRealPointFlux * point_flux_driver = new GSeaRealPointFlux(fGenPar);

  // set rotation for coordinate tranformation from the topocentric horizontal
  // system to a user-defined coordinate system:
  if(!fGenPar->Rotation.IsIdentity()) {
     point_flux_driver->SetUserCoordSystem(fGenPar->Rotation);
  }
  
  fGenPar->GlobalGenWeight=point_flux_driver->GetGlobalGenWeight();
  
  return point_flux_driver;
}
//________________________________________________________________________________________
int GGenerateEvent::PosInCan(double VertX, double VertY, double VertZ){
  // check if the interaction vertex is inside the detector can 

  int VerInCan = 0;

  if (TMath::Sqrt(VertX*VertX+VertY*VertY)<=fGenPar->Can3) {
    if(VertZ>= fGenPar->Can1 && VertZ<=fGenPar->Can2)VerInCan = 1;
    else if(VertZ>= fGenPar->Can1-fGenPar->HBedRock && VertZ<fGenPar->Can1)VerInCan = 2;
  
  }

  return VerInCan;
}
//________________________________________________________________________________________
void GGenerateEvent::FillEvent()
{

  // just copy the thing.
  fSeaEvent->Tracks = gWritenTracks;
  fSeaEvent->NTracks=fSeaEvent->Tracks.size();  

}
//________________________________________________________________________________________
bool GGenerateEvent::Propagation(GSeaTrack *t)
{
  // wrapper function to call propagation with regular GSeaTrack

  PropaTrack pt;
  copy( *t, pt );
  bool r = Propagation ( &pt );
  copy( pt, *t );
  return r;
}
//________________________________________________________________________________________
bool GGenerateEvent::Propagation(PropaTrack * trackf){

  PropaTrack track=*trackf;  

  int iFlagCan;
  bool LepCan=false;
  double DepthW,DepthR;

  double range=pow(10.,fGenPar->RangeSW.Eval(log10(track.E)))/(fGenPar->RhoSW);

#ifdef _MUSIC_ENABLED__    
  int MUMODEL2=1;
  double DepthsF;
  double RhoSW=fGenPar->RhoSW;
  double RhoSR=fGenPar->RhoSR;
#endif

  if(track.E<0.16)return false; // muon at rest (no propagation)

// We want to know if the muon track intercepts the can

  iFlagCan=DistToCan(track);

  if(iFlagCan==1)LepCan = true; 			// the track is inside the can (no propagation)  
  else if(iFlagCan==2 || iFlagCan==-1)LepCan = false;   // the track does not intercept the can (no propagation)
  else if(range<fDistaCan)LepCan = false;		// muon does not have chance to reach the can (no propagation)
  else {
  
    while(1){
      
      bool InRock=false;
      DepthR=fabs((fGenPar->Can1-track.V3)/(track.D3));
      DepthW=fDistaCan;
      
      if(track.V3<=fGenPar->Can1){
        if(DepthR>0.01)InRock=true;
	else InRock=false;	
      }

      if(InRock){			// I'm in rock

    // Transport in rock the muon
        if(fGenPar->PropCode.compare("PropaMuon")==0){      
          if(!PropaMuonSR->Propagate(track,DepthR)){LepCan=false; break;}
           *trackf = PropaMuonSR->GetPropaTrack();  
        }
#ifdef _MUSIC_ENABLED__       
        else if(fGenPar->PropCode.compare("MUSIC")==0){
          DepthR*=100.;
          track.V1*=100.;
          track.V2*=100.;
          track.V3*=100.;     
          muon_transport_sr_(&track.V1,&track.V2,&track.V3,&track.D1,&track.D2,&track.D3,&track.E,&DepthR,&track.T,&RhoSR,&MUMODEL2,&MUMODEL2,&trackf->V1,&trackf->V2,&trackf->V3,&trackf->D1,&trackf->D2,&trackf->D3,&trackf->E,&DepthsF,&trackf->T);
	  trackf->V1/=100.;
          trackf->V2/=100.;
          trackf->V3/=100.;   
        }      
#endif  	
#ifdef _PROPOSAL_ENABLED__   
        else if(fGenPar->PropCode.compare("PROPOSAL")==0){
	  ProposalSR->GetParticle()->SetEnergy(track.E*1E3);	// in Mev	
	  ProposalSR->GetParticle()->SetX(track.V1*1E2);	// in cm
	  ProposalSR->GetParticle()->SetY(track.V2*1E2);	// in cm
	  ProposalSR->GetParticle()->SetZ(track.V3*1E2);	// in cm
	  ProposalSR->GetParticle()->SetT(track.T);	// in ?
	  ProposalSR->GetParticle()->SetTheta(acos(track.D3)*180./kPi);	
	  ProposalSR->GetParticle()->SetPhi(atan2(track.D2,track.D1)*180./kPi);		
          ProposalSR->GetParticle()->SetPropagatedDistance(0);
	  ProposalSR->Propagate(DepthR*100.);		  
	  trackf->E=ProposalSR->GetParticle()->GetEnergy()/1.E3;	
	  trackf->V1=ProposalSR->GetParticle()->GetX()/1.E2;
	  trackf->V2=ProposalSR->GetParticle()->GetY()/1.E2;
	  trackf->V3=ProposalSR->GetParticle()->GetZ()/1.E2;
	  trackf->T=ProposalSR->GetParticle()->GetT();	
	  trackf->D1=ProposalSR->GetParticle()->GetSinTheta()*ProposalSR->GetParticle()->GetCosPhi();
	  trackf->D2=ProposalSR->GetParticle()->GetSinTheta()*ProposalSR->GetParticle()->GetSinPhi();
	  trackf->D3=ProposalSR->GetParticle()->GetCosTheta();
        }
#endif  
      }
      else{	

  	// Transport in sea water the muon
        if(fGenPar->PropCode.compare("PropaMuon")==0){      
          if(!PropaMuonSW->Propagate(track,DepthW)){LepCan=false; break;} 
           *trackf = PropaMuonSW->GetPropaTrack();  
        }
#ifdef _MUSIC_ENABLED__       
        else if(fGenPar->PropCode.compare("MUSIC")==0){
          DepthW*=100.;
          track.V1*=100.;
          track.V2*=100.;
          track.V3*=100.;     
          muon_transport_sw_(&track.V1,&track.V2,&track.V3,&track.D1,&track.D2,&track.D3,&track.E,&DepthW,&track.T,&RhoSW,&MUMODEL2,&MUMODEL2,&trackf->V1,&trackf->V2,&trackf->V3,&trackf->D1,&trackf->D2,&trackf->D3,&trackf->E,&DepthsF,&trackf->T);
          trackf->V1/=100.;
          trackf->V2/=100.;
          trackf->V3/=100.;     
        }
#endif  
#ifdef _PROPOSAL_ENABLED__   
        else if(fGenPar->PropCode.compare("PROPOSAL")==0){
	  ProposalSW->GetParticle()->SetEnergy(track.E*1E3);	// in Mev	
	  ProposalSW->GetParticle()->SetX(track.V1*1E2);	// in cm
	  ProposalSW->GetParticle()->SetY(track.V2*1E2);	// in cm
	  ProposalSW->GetParticle()->SetZ(track.V3*1E2);	// in cm
	  ProposalSW->GetParticle()->SetT(track.T);	// in ?
	  ProposalSW->GetParticle()->SetTheta(acos(track.D3)*180./kPi);	
	  ProposalSW->GetParticle()->SetPhi(atan2(track.D2,track.D1)*180./kPi);		
          ProposalSW->GetParticle()->SetPropagatedDistance(0);
	  ProposalSW->Propagate(DepthW*100.);	
	  trackf->E=ProposalSW->GetParticle()->GetEnergy()/1.E3;	
	  trackf->V1=ProposalSW->GetParticle()->GetX()/1.E2;
	  trackf->V2=ProposalSW->GetParticle()->GetY()/1.E2;
	  trackf->V3=ProposalSW->GetParticle()->GetZ()/1.E2;	
	  trackf->T=ProposalSW->GetParticle()->GetT();	
	  trackf->D1=ProposalSW->GetParticle()->GetSinTheta()*ProposalSW->GetParticle()->GetCosPhi();
	  trackf->D2=ProposalSW->GetParticle()->GetSinTheta()*ProposalSW->GetParticle()->GetSinPhi();
	  trackf->D3=ProposalSW->GetParticle()->GetCosTheta();	
        }
#endif 
      }
  
      if(trackf->E<0.16){LepCan=false; break;}
      track=*trackf;
  
      iFlagCan=DistToCan(track);
      
      if(fDistaCan>0 && fDistaCan<0.01){LepCan=true; break;}
      if(iFlagCan==1){LepCan=true; break;} // the track is inside the can (no propagation)  
      else if(iFlagCan==2 || iFlagCan==-1){LepCan=false; break;} 
  
    }  // close while(1)

  }
  
  if(LepCan) trackf->Status=1;
  
  return LepCan;

}
//___________________________________________________________________________
int GGenerateEvent::DistToCan(PropaTrack track)
{

// iFlag = -1 something has gone wrong!
// iFlag =  0 if the track intercepts the can AND the range is positive
// iFlag =  1 if the point is inside the can
// iFlag =  2 if the track never intercepts the can

  int iFlag=-1;
  fDistaCan = -1;    
  
  double Zmin= fGenPar->Can1;
  double Zmax= fGenPar->Can2;  
  double Radius = fGenPar->Can3;

  double Vx = track.V1;
  double Vy = track.V2;
  double Vz = track.V3;
  
  double Dx = track.D1;
  double Dy = track.D2;
  double Dz = track.D3;

  double utit=sqrt(Dx*Dx+Dy*Dy+Dz*Dz);

  Dx/=utit;
  Dy/=utit;
  Dz/=utit;

  fDistaX=0.; fDistaY=0.; fDistaZ=0.;
  
  if(Vz<=Zmax && Vz>=Zmin && Vx*Vx+Vy*Vy<=Radius*Radius){
    iFlag = 1;
    fDistaCan = 0.;
  }
  else if(fabs(Dz)==1){
    if (sqrt(Vx*Vx+Vy*Vy) > Radius) iFlag = 2;
    else if(Dz==1 && Vz>Zmax) iFlag = 2;    
    else if(Dz==-1 && Vz<Zmin) iFlag = 2;
    else{
      iFlag =  0;      
      if(Dz==1){
        fDistaX=Vx;
	fDistaY=Vy;
	fDistaZ=Zmin;
        fDistaCan = fabs(Zmin-Vz);
      }
      else {
        fDistaX=Vx;
	fDistaY=Vy;
	fDistaZ=Zmax;        
	fDistaCan = fabs(Vz-Zmax);  
      }
    } 
  }
  else {
    double a = Dx*Dx+Dy*Dy;
    double b = 2.*(Vx*Dx+Vy*Dy);
    double c = Vx*Vx+Vy*Vy-Radius*Radius;
    double delta=b*b-4.*a*c;
    if(delta<0)iFlag = 2;
    else {
      delta = sqrt(delta);
      double t1=(-b-delta)/(2.*a);
      double t2=(-b+delta)/(2.*a);
      double z1=Vz+Dz*t1;
      double z2=Vz+Dz*t2;
      if((z1<Zmin && z2<Zmin) || (z1>Zmax && z2>Zmax)) iFlag = 2;
      else if(z1>=Zmin && z1<=Zmax){
        if(t1>=0){
          iFlag = 0;
	  fDistaX=Vx+Dx*t1;
	  fDistaY=Vy+Dy*t1;
	  fDistaZ=Vz+Dz*t1;
          fDistaCan = fabs(t1); 
	} else iFlag = 2;	// intersection before the vertex	
      }      
      else if(z1<Zmin && Dz>0){  // up-going
        iFlag = 0;
	t1=(Zmin-Vz)/Dz;
	if(t1>=0){
	  fDistaX=Vx+Dx*t1;
	  fDistaY=Vy+Dy*t1;
	  fDistaZ=Vz+Dz*t1;
          fDistaCan=fabs((Zmin-Vz)/Dz);
	} else iFlag = 2;	// intersection before the vertex      
      }
      else if(z1>Zmax && Dz<0){  // down-going
        iFlag = 0;
	t1=(Zmax-Vz)/Dz;
	if(t1>=0){	
	  fDistaX=Vx+Dx*t1;
	  fDistaY=Vy+Dy*t1;
	  fDistaZ=Vz+Dz*t1;
	  fDistaCan=fabs((Zmax-Vz)/Dz);
	} else iFlag = 2;	// intersection before the vertex
      }
    }    
  }
  
//  if(iFlag==0)cout<<"DistTotCan "<<iFlag<<" "<<fDistaCan<<" "<<fDistaX<<" "<<fDistaY<<" "<<fDistaZ<<" "<<Vx<<" "<<Vy<<" "<<Vz<<" "<<Dx<<" "<<Dy<<" "<<Dz<<endl;
  
  return iFlag;
}
//________________________________________________________________________________________________________
bool GGenerateEvent::PropagateTracks(vector<GSeaTrack> GeneratorTracks, int VerInCan) {

//Move all the muons to the can and add them to gWriteTracks

  gPropagatedTracks.clear();
     
  // find the highest index so far...
  unsigned nextId = 0;
  for ( auto& track : GeneratorTracks ) nextId = std::max( nextId, track.Id +1 );

  //first propagate the muons that are outside the can
  int nGenTracks = GeneratorTracks.size();
  gPropagatedTracks = GeneratorTracks;
  bool LepInCan = false;
  if (VerInCan!=0) LepInCan = true;
  else  {
    int aux=0;
    for ( auto& track : gPropagatedTracks ) {
      if (aux >= nGenTracks) break;
      if( abs(track.PdgCode)==13 && track.Status==1 && track.InCan==0 ) { //propagate final state muons that are outside can
        auto propagated_track = track;
        if ( Propagation( &propagated_track ) ){
          track.Status += 1000;              //Status > 1000 means: there is a daughter that is propagated
          propagated_track.MotherId = track.Id;
          propagated_track.InCan    = 1;
          propagated_track.Id       = nextId++; // post-increment
          gPropagatedTracks.push_back( propagated_track );
          LepInCan = true;
        }
      }
      else if ( track.InCan!=0 ) LepInCan = true;
      aux++;
    }
  }

  return LepInCan;

}
//________________________________________________________________________________________________________
bool GGenerateEvent::WriteTracks(bool LepInCan) {

//Add particles to the gWriteTracks depending on WriteParticlesMode.

  gWritenTracks.clear();

  if      ( fGenPar -> WriteParticlesMode == 2 ) { gWritenTracks = gPropagatedTracks; }
  else if ( fGenPar -> WriteParticlesMode == 1 ) { if (LepInCan) gWritenTracks = gPropagatedTracks; }
  else if ( fGenPar -> WriteParticlesMode == 0 ) {
    if (LepInCan) {
      // writes particles according to the mode
      map<int, int> family_map;
      for ( auto& track : gPropagatedTracks ) {
        bool write_me = (track.Status % 1000) < 2 ; // appendix d of genie manual
        // if certain (unstable) particles are not written to the output, we change the parent-child relation accordingly.
        if ( !write_me ) family_map [ track.Id ] = track.MotherId; // Leave the kids with grandma
        else {
          while (1) {
            // Did mommy leave me with grandma?
            auto id = family_map.find( track.MotherId );
            if ( id == family_map.end() ) break;
            // addopt grandma as my mother
            track.MotherId = id->second;
            // set 3rd decimal digit of status to 1 to indicate
            // the track once had an intermediate parent.
            bool is_orphan = (track.Status % 1000 ) / 100 > 1;
            if ( !is_orphan ) track.Status += 100;
          } // loop bcause grandma may be gone too
        }
        if( write_me ) gWritenTracks.push_back(track);
      }
    }
  }

  return gWritenTracks.size() > 0;

}
//___________________________________________________________________________
double GGenerateEvent::GetXSecWater(int neutrino_pdg, double Energy){
  
  double XSec=0.;

  map<int,TGraph>::iterator iter = fGenPar->XSecWater.begin();
  for ( ; iter != fGenPar->XSecWater.end(); ++iter) {
    if(neutrino_pdg==iter->first){        
       TGraph GrSecTot = iter->second;	
       XSec=GrSecTot.Eval(Energy);	
       break;
    }    
  }

  return XSec;
}
//___________________________________________________________________________
double GGenerateEvent::GetXSecSW(int neutrino_pdg, double Energy){
  
  double XSec=0.;

  map<int,TGraph>::iterator iter = fGenPar->XSecTotSW.begin();
  for ( ; iter != fGenPar->XSecTotSW.end(); ++iter) {
    if(neutrino_pdg==iter->first){        
       TGraph GrSecTot = iter->second;  
       XSec=GrSecTot.Eval(Energy);  
       break;
    }    
  }

  return XSec;
}
