//____________________________________________________________________________
/*!
 
\class   genie::GGenerateEvent

\brief   A base generation driver for an under-water neutrino telescope

\author  Carla Distefano <distefano_c@lns.infn.it>
         LNS-INFN, Catania
         Included modifications by Alfonso Garcia <alfonsog@nikhef.nl>
	 NIKHEF, Amsterdam (January 2019)

\created July 19, 2017

\cpright  Copyright (c) 2017-2019, The KM3NeT Collaboration
          For the full text of the license see $GSEAGEN/LICENSE

*/
//____________________________________________________________________________

#ifndef _GGENERATEEVENT_
#define _GGENERATEEVENT_

#include <string>
#include <map>
#include <vector>
#include <cstdio>
#include <iostream>
#include <fstream>

#include <TSystem.h>
#include <TRotation.h>

#include "SeaEvent/GBinParam.h"
#include "SeaEvent/GSeaEvent.h"

#include "SeaNuDrivers/GSeaRealAtmoFlux.h"
#include "SeaNuDrivers/GSeaRealPointFlux.h"
#include "SeaNuDrivers/GSeaAntaresFileFlux.h"
#include "SeaNuDrivers/GSeaFileFlux.h"

#include "PropaMuon/PropaMuon.h"

#ifdef _PROPOSAL_ENABLED__    
#include "PROPOSAL/Propagator.h"
#include "PROPOSAL/Output.h"
#endif

#ifdef _MUSIC_ENABLED__   
#include <boost/program_options/options_description.hpp>
#endif

using namespace std;
using namespace genie;
using namespace genie::flux;
using namespace genie::constants;



namespace genie {

class GGenerateEvent {

public :

  GGenerateEvent(GenParam * GenPar);
  virtual ~GGenerateEvent();
  
  virtual vector<GSeaTrack> 	GetGeneratorTracks (void) = 0;
  
  
  virtual void			Initialize	   (void) = 0;	
  virtual void                  Configure          (int NTot) = 0;	
  virtual double		GlobProbScale      (void) = 0;	
  
  virtual void 			SetGeometry	   (string RootGeomFile) = 0;	
  
  virtual TLorentzVector        GetVertex          (void) = 0;	
  virtual double 		GetGlobalGenWeight (void) = 0;	
  
  virtual void			FillSeaEvent       (GSeaEvent * SeaEvent) = 0;	
  virtual void  		ComputeWeights     (GSeaEvent * SeaEvt) = 0;	
  virtual void			InitGenBin	   (double Emin, double Emax, double RL, double RT) = 0;	
  virtual void			InitGenBin	   (double Emin, double Emax, double RL, double RT, double X0, double Y0, double Z0) = 0;	
  virtual double 		GetNFluxNeutrinos  (void) = 0;	
  
  virtual void 			GenerateEvent      (void) = 0;	
  virtual void			WriteNative	   (GSeaEvent * SeaEvt) = 0;	
  virtual void			CleanEvent	   (void) = 0;	

  virtual void                  CalcXSecWater      (void) = 0;	
  virtual map<int, TGraph>      CalcXsecTot        (map<int, double> MediaComp) = 0;  


  GenParam  *			fGenPar;	
  GSeaEvent *			fSeaEvent;	

#ifdef _ANTARES_ENABLED__  
  event *			fAntaresEvt;
#endif  
  
  PropaMuon *                   PropaMuonSW;	
  PropaMuon *                   PropaMuonSR;	

#ifdef _PROPOSAL_ENABLED__    
  Propagator *			ProposalSW;	
  Propagator *			ProposalSR;	
#endif

  vector<GSeaTrack> 		gPropagatedTracks;
  vector<GSeaTrack> 		gWritenTracks;
 
  
  int 				fNEvt;	
  int 				fNTot;	
  double 			fDistaCan;	
  double			fDistaX,fDistaY,fDistaZ;	

  GSeaRealAtmoFlux * 		flux_driver_diffuse;	
  GSeaRealPointFlux * 		flux_driver_point;	
  GSeaAtmoFlux * 		fFluxDriver;  
  GSeaFileFlux *		fFileDriver;	

  bool				EoF;			// check if end of file if neutrinos rad from input file

  GSeaRealAtmoFlux *   		GetFluxDiffuse     (void);	
  GSeaRealPointFlux *   	GetFluxPoint	   (void);	
  
  int 				PosInCan           (double VertX, double VertY, double VertZ);

  bool    			Propagation        (GSeaTrack *t);
  bool                          Propagation        (PropaTrack * trackf);	

  int                           DistToCan	   (PropaTrack track);	

  bool    			PropagateTracks    (vector<GSeaTrack> GeneratorTracks, int VerInCan);
  bool				WriteTracks        (bool LepInCan);				

  void 				FillEvent	   (void);

  double			GetXSecWater	   (int neutrino_pdg, double Energy);	
  double  			GetXSecSW          (int neutrino_pdg, double Energy);

  protected:

  
};

} // genie namespace


#endif // _GGENERATEEVENT_