//____________________________________________________________________________
/*! 

\class   genie::flux::GSeaAntaresFileFlux

\brief   A flux driver to read incident neutrinos from evt files

\author  Carla Distefano <distefano_c@lns.infn.it>
         LNS-INFN, Catania

\created March 18, 2019

\cpright  Copyright (c) 2019, The KM3NeT Collaboration
          For the full text of the license see $GSEAGEN/LICENSE

*/
//____________________________________________________________________________

#ifdef _ANTARES_ENABLED__


#include <fstream>
#include <cassert>
#include <cstdio>
#include <iostream>
#include <fstream>

#include <TH2D.h>
#include <TH3D.h>
#include <TMath.h>
#include <TF1.h>
#include <TF3.h>
#include <TFile.h>

#ifdef _GENIEVERSIONGTEQ3__
#include "Framework/Conventions/Constants.h"
#include "Framework/Messenger/Messenger.h"
#include "Framework/ParticleData/PDGLibrary.h"
#else
#include "Conventions/Constants.h"
#include "Messenger/Messenger.h"
#include "PDG/PDGLibrary.h"
#endif

#include "SeaNuDrivers/GSeaAntaresFileFlux.h"


using namespace std;
using namespace genie;
using namespace genie::flux;
using namespace genie::constants;




//___________________________________________________________________________
GSeaAntaresFileFlux::GSeaAntaresFileFlux(GenParam * GenPar) :
GSeaFileFlux(GenPar)
{

  fEvt = new event();
  
  if(fGenPar->FluxFiles.size()>1){
    LOG("GSeaAntaresFileFlux", pFATAL) << "Only one input file is allowed, exiting...";
    exit(1);
  }
  
  if(fGenPar->FluxFiles[0].FileNames.size()>1){
    LOG("GSeaAntaresFileFlux", pFATAL) << "Only one input file is allowed, exiting...";
    exit(1);
  }
  
  for(unsigned i=0; i<fGenPar->NuList.size(); i++)fPdgCList->push_back(fGenPar->NuList[i]); 

  fInFile.open(fGenPar->FluxFiles[0].FileNames[0]);
  if(!fInFile.is_open()){
    LOG("GSeaAntaresFileFlux", pFATAL) << "Error in opening file "<<fGenPar->FluxFiles[0].FileNames[0]<<", exiting...";
    exit(1);
  }
  
  

  this->LoadHeaderFile();   
  
  
    



  // Configure GFileFlux options
  // set min/max energy:
  this->SetMinEnergy (fGenPar->EvMin * units::GeV);
  this->SetMaxEnergy (fGenPar->EvMax * units::GeV);    
  
  // set min/max costheta:
  this->SetMinCosTheta (fGenPar->CtMin);
  this->SetMaxCosTheta (fGenPar->CtMax);    


  LOG("GSeaAntaresFileFlux", pNOTICE)
       << "Instantiating the Antares file flux driver";
 
 
} 
//___________________________________________________________________________
GSeaAntaresFileFlux::~GSeaAntaresFileFlux()
{

  fInFile.close();

}
//___________________________________________________________________________
void GSeaAntaresFileFlux::LoadHeaderFile()
{

  bool new_r, new_evt;
  unsigned int nRun, nEvt, Type;

  fEvt->read(fInFile);
  fEvt->info(new_r, new_evt, nRun, nEvt, Type);

  if(new_r){
    
    fGenPar->RunNu=nRun;
  
    unsigned nLine=fEvt->ndat("cut_seamuon");	  
    for(unsigned i=0; i<nLine; i++){      
      string line = fEvt->next("cut_seamuon");  
//      sscanf(line.c_str(),"%lf %lf %lf %lf",&fGenPar->EvMin,&fGenPar->EvMax,&fGenPar->CtMin,&fGenPar->CtMax); 	 
      
	// dobbiamo controllare che emin e emax sono validi....      
             
    }  
  
  }
  else{
    LOG("GSeaAntaresFileFlux", pFATAL) << "Evt file does not have a header, exiting...";
    exit(1);
  }

  fMinEv = fGenPar->EvMin;            
  fMaxEv = fGenPar->EvMax;            
  fCosThetaMin = fGenPar->CtMin;      
  fCosThetaMax = fGenPar->CtMax;      
  fPhiMin = 0.;           
  fPhiMax = 2.*kPi;           

  this->SetMinEnergy(fGenPar->EvMin);
  this->SetMaxEnergy(fGenPar->EvMax);  
  this->ResetNFluxNeutrinos();

  return;
}

//___________________________________________________________________________
void GSeaAntaresFileFlux::ReadFileEvent()
{

  bool new_r, new_evt;
  unsigned int nRun, nEvt, Type;

  unsigned nLine;
  string line;

  Neutrinos.clear();
  Muons.clear();

  double RT=sqrt(pow((fGenPar->Can2-fGenPar->Can1),2)+pow(fGenPar->Can3*2,2))/2.;
  double RL=0.;



  RandomGen * rnd = RandomGen::Instance();

  if(fEvt->read(fInFile)==0){

    fEvt->info(new_r, new_evt, nRun, nEvt, Type);

    if(!new_r){

      nLine=fEvt->ndat("track_primary");	
      
      double Vx,Vy,Vz,Dx,Dy,Dz,E,T;
      int Id,Code;

      line=fEvt->next("track_primary"); 
      sscanf(line.c_str(),"%d %lf %lf %lf %lf %lf %lf %lf %lf %d ",&Id,&Vx,&Vy,&Vz,&Dx,&Dy,&Dz,&E,&T,&Code); 	 

      double Xc=0.;
      double Yc=0.;
      double Zc=-fGenPar->SeaBottomRadius+fGenPar->Can1;  // in m
      
      double R=fGenPar->SeaBottomRadius+fGenPar->SiteDepth;
    
      double a=1.;
      double b=2.*(Dx*(fX0-Xc)+ Dy*(fY0-Yc) +Dz*(fZ0-Zc));
      double c=pow(fX0-Xc,2)+pow(fY0-Yc,2)+pow(fZ0-Zc,2)-R*R;
      
      double Delta=b*b-4*a*c;
      double t=(-b-sqrt(Delta))/(2.*a);
      
//      RT=fRt;
      RL=fabs(t);
      
      double DistaMax=0.;
      double Dista;

      nLine=fEvt->ndat("track_seaneutrino");	
      for(unsigned i=0; i<nLine; i++){ 
        GSeaTrack Neutrino; 
        line = fEvt->next("track_seaneutrino"); 
        sscanf(line.c_str(),"%d %lf %lf %lf %lf %lf %lf %lf %lf %d ",&Neutrino.Id,&Neutrino.V1,&Neutrino.V2,&Neutrino.V3,&Neutrino.D1,&Neutrino.D2,&Neutrino.D3,&Neutrino.E,&Neutrino.T,&Neutrino.PdgCode); 	 
        if(Neutrino.PdgCode==66)Neutrino.PdgCode=12;
        else if(Neutrino.PdgCode==67)Neutrino.PdgCode=-12;
        else if(Neutrino.PdgCode==68)Neutrino.PdgCode=14;
        else if(Neutrino.PdgCode==69)Neutrino.PdgCode=-14;
        else if(Neutrino.PdgCode==133)Neutrino.PdgCode=16;
        else if(Neutrino.PdgCode==134)Neutrino.PdgCode=-16;
	
	Dista=sqrt(pow(Neutrino.V1,2)+pow(Neutrino.V2,2)+pow(Neutrino.V3,2));
	if(Dista>DistaMax)DistaMax=Dista;
	if(Neutrino.E>fGenPar->EvMin&& Neutrino.E<fGenPar->EvMax)Neutrinos.push_back(Neutrino);
      }
      
      nLine=fEvt->ndat("track_seamuon");	
      for(unsigned i=0; i<nLine; i++){ 
        GSeaTrack Muon;
	line = fEvt->next("track_seamuon"); 
        sscanf(line.c_str(),"%d %lf %lf %lf %lf %lf %lf %lf %lf %d ",&Muon.Id,&Muon.V1,&Muon.V2,&Muon.V3,&Muon.D1,&Muon.D2,&Muon.D3,&Muon.E,&Muon.T,&Muon.PdgCode); 	 
        if(Muon.PdgCode==5)Muon.PdgCode=-13;
        else if(Muon.PdgCode==6)Muon.PdgCode=13;
	Dista=sqrt(pow(Muon.V1,2)+pow(Muon.V2,2)+pow(Muon.V3,2));
	if(Dista>DistaMax)DistaMax=Dista;	
	Muons.push_back(Muon);
	
	
      }
      
//      fEvt->tagd("track_seamuon");
      
      
      RT+=DistaMax;
      
      nLine=fEvt->ndat("weights");
      for(unsigned i=0; i<nLine; i++){
        double w1,w2,w3;
        line = fEvt->next("weights");
	sscanf(line.c_str(),"%lf %lf %lf",&w1,&w2,&w3);    
	w1=kPi*RT*RT;
	w2*=w1;
	w3*=w1;
	fEvt->tagd("weights", i);
	char ch[1024];
	sprintf(ch, "%12.3E %12.3E %12.3E",w1,w2,w3);
	fEvt->taga("weights",ch);
	
	sprintf(ch, "%f",DistaMax);
	fEvt->taga("dista_max",ch);
	
	sprintf(ch, "%f",RT);
	fEvt->taga("RADD",ch);

      }

      
      // generate the primary vertex 

      double x = 0.;
      double y = 0.;
      double z = 0.;  
	
      x += -RL * Dx;  // we have to multiply RL times the arrival direction cosines
      y += -RL * Dy;	
      z += -RL * Dz;
	
      TVector3 vec(x,y,z);	   		// vector towards selected point
      TVector3 dvec1 = vec.Orthogonal(); 	// orthogonal vector
      TVector3 dvec2 = dvec1;		   	// second orthogonal vector
        
      dvec2.Rotate(-kPi/2.0,vec);	 // rotate second vector by 90deg, now forming a new orthogonal cartesian coordinate system
        
      double psi = 2.*kPi* rnd->RndFlux().Rndm(); // rndm angle [0,2pi]
      double random = rnd->RndFlux().Rndm();	    // rndm number  [0,1]
        
      dvec1.SetMag(TMath::Sqrt(random)*RT*TMath::Cos(psi));
      dvec2.SetMag(TMath::Sqrt(random)*RT*TMath::Sin(psi));
        
      x += dvec1.X() + dvec2.X();
      y += dvec1.Y() + dvec2.Y();
      z += dvec1.Z() + dvec2.Z();

      x += fX0;
      y += fY0;
      z += fZ0;


      for(unsigned i=0; i<Neutrinos.size(); i++){        
        Neutrinos[i].V1+=x;
        Neutrinos[i].V2+=y;
        Neutrinos[i].V3+=z;
      }


      for(unsigned i=0; i<Muons.size(); i++){  
      
        Muons[i].V1+=x;
        Muons[i].V2+=y;
        Muons[i].V3+=z;

        // for muons, we translated the starting point from the generation area up to the sealevel

        a=1.;
        b=2.*(Muons[i].D1*(Muons[i].V1-Xc)+ Muons[i].D2*(Muons[i].V2-Yc) +Muons[i].D3*(Muons[i].V3-Zc));
        c=pow(Muons[i].V1-Xc,2)+pow(Muons[i].V2-Yc,2)+pow(Muons[i].V3-Zc,2)-R*R;
      
        Delta=b*b-4*a*c;
        t=(-b-sqrt(Delta))/(2.*a);
      
        Muons[i].V1+=t*Muons[i].D1;
        Muons[i].V2+=t*Muons[i].D2;
        Muons[i].V3+=t*Muons[i].D3;
      
      }
    }
  }
  else 
    EoF=true;

  return;
}

#endif
